#include <SimpleDHT.h>
#include <LiquidCrystal_I2C.h>
#include <Adafruit_BMP085.h>
#include "DFRobot_CCS811.h"

#define CO2_THRESHOLD 5000

// pin for dht11
int pinDHT11 = 2;

// pin for buzzer
int buzzer = 12;

// dht11 object
SimpleDHT11 dht11;

// lcd object
LiquidCrystal_I2C lcd(0x27, 16, 2);

// CCS811 object
DFRobot_CCS811 CCS811;

// BMP180 object
Adafruit_BMP085 bmp;

// used to represent alarm on/off state
bool alarmActivated;

float calculateRealTemperature(float measuredTemperature, float humidity, float pressure) {
  // Constants for ideal gas law
  const float R = 8.314;  // Universal gas constant (J/(mol·K))
  const float M = 28.97;  // Molar mass of dry air (g/mol)

  // Convert humidity from percentage to decimal fraction
  float relativeHumidity = humidity / 100.0;

  // Calculate vapor pressure
  float e = relativeHumidity * 6.112 * exp(17.67 * measuredTemperature / (measuredTemperature + 243.5));

  // Calculate partial pressure of dry air
  float pd = pressure - e;

  // Calculate the number of moles of dry air
  float n = pd / (R * (measuredTemperature + 273.15));

  // Calculate the real temperature using the ideal gas law
  float realTemperature = measuredTemperature + ((e * M) / (R * n));

  return realTemperature;
}

void setup() {
  Serial.begin(9600);

  // init lcd
  lcd.begin();
  lcd.backlight();

  // init CCS811
  while(CCS811.begin() != 0){
    Serial.println("failed to init chip, please check if the chip connection is fine");
  }

  // init BMP180
  while(!bmp.begin()) {
    Serial.println("Could not find a valid BMP085 sensor, check wiring!");
  }

  // setup buzzer
  pinMode(buzzer, OUTPUT);

  // initial state of alarm
  alarmActivated = true;
}

void loop() {
  Serial.println("=================================");

  // ccs811
  if(CCS811.checkDataReady()) {
    // threshold value from https://www.kane.co.uk/knowledge-centre/what-are-safe-levels-of-co-and-co2-in-rooms

    // if CO2 over threshold, activate alarm
    if (CCS811.getCO2PPM() >= CO2_THRESHOLD) {
      digitalWrite(buzzer, HIGH);
      lcd.clear();
      lcd.print("CO2 at threshold");
      alarmActivated = true;
    }
    else {
      digitalWrite(buzzer, LOW);
      alarmActivated = false;
    }
    Serial.print("CO2: ");
    Serial.print(CCS811.getCO2PPM());
    Serial.print("ppm, TVOC: ");
    Serial.print(CCS811.getTVOCPPB());
    Serial.println("ppb");

    CCS811.writeBaseLine(0x447B);
  }
  
  if (!alarmActivated) {
    // read, store and print DHT11 data
    float temperature = 0;
    float humidity = 0;
    byte data[40] = {0};
    if (dht11.read2(pinDHT11, &temperature, &humidity, data)) {
      Serial.print("Read DHT11 failed");
      return;
    }

    Serial.print("DHT11: ");
    Serial.print(temperature); Serial.print(" *C, ");
    Serial.print(humidity); Serial.println(" %");

    // // read, store and print BMP180 data
    float bmpTemp = bmp.readTemperature();

    Serial.print("Temperature = ");
    Serial.print(bmpTemp);
    Serial.println(" *C");
    
    float readPressure = bmp.readPressure();
    Serial.print("Pressure = ");
    Serial.print(bmp.readPressure());
    Serial.println(" Pa");
    
    Serial.print("Altitude = ");
    Serial.print(bmp.readAltitude());
    Serial.println(" meters");

    Serial.print("Pressure at sealevel (calculated) = ");
    Serial.print(bmp.readSealevelPressure());
    Serial.println(" Pa");

    Serial.print("Real altitude = ");
    Serial.print(bmp.readAltitude(101500));
    Serial.println(" meters");
    Serial.println();

    float realTemp = calculateRealTemperature(bmpTemp, humidity, readPressure);

    // print temperature measured by sensor
    lcd.setCursor(0, 0);
    lcd.print("Sensor: "); lcd.print(bmpTemp); lcd.print(" *C");

    // print real temperature
    lcd.setCursor(0, 1);
    lcd.print("Real:   "); lcd.print(realTemp); lcd.print(" *C");
  }

  delay(2000);
}
